// Copyright (c) 2024 Infineon Technologies AG. All rights reserved
// Licensed under the Apache License, Version 2.0, see APACHE_LICENSE for details.
// SPDX-License-Identifier: Apache-2.0

#include "cy_pdl.h"
#include "cybsp.h"


/*******************************************************************************
* Macros
*******************************************************************************/

/*******************************************************************************
* Global Variables
*******************************************************************************/


/*******************************************************************************
* Function Prototypes
*******************************************************************************/


/*******************************************************************************
* Function Definitions
*******************************************************************************/


/*******************************************************************************
* Function Name: main
********************************************************************************
* Summary:
* This is the main function for the CPU. It configures the PWM and puts the CPU
* in Sleep mode to save power.
*
* Parameters:
*  void
*
* Return:
*  int
*
*******************************************************************************/
int main(void)
{
    /* API return code */
    cy_rslt_t result;

    /* Initialize the device and board peripherals */
    result = cybsp_init();
    if (result != CY_RSLT_SUCCESS)
    {
        CY_ASSERT(0);
    }

    /* Enable global interrupts */
    __enable_irq();

    /* Initialize the PWM */
    Cy_TCPWM_PWM_Init(PWM_LED1_HW, PWM_LED1_NUM, &PWM_LED1_config);
        
        /* Enable the initialized PWM */
    Cy_TCPWM_PWM_Enable(PWM_LED1_HW, PWM_LED1_NUM);
        
        /* Then start the PWM */
    Cy_TCPWM_TriggerStart_Single(PWM_LED1_HW, PWM_LED1_NUM);

    for (;;)
    {
        /* Put the CPU into sleep mode to save power */
        Cy_SysPm_Sleep(0UL);
    }
}


/* [] END OF FILE */